#!/usr/bin/tclsh

set version 1.0
set version 1.1
set version 2.0
set version 2.1  ;# extract booth
set version 2.2  ;# mark deleted games
set version 3.0  ;# extract bgglink again; add language column
set version 3.1  ;# remove empty lines with &nbsp;
set version 3.2  ;# corrected lang and prio columns
set version 3.3  ;# print spielbox link in first line
set version 4.0  ;# extract spielen.de link
set version 5.0  ;# column for price
set version 5.1  ;# do not mark games with remark "BGG" as deleted
set version 5.2  ;# read bgg website to
set version 5.3  ;# bugfix for bgg board game extraction
set version 5.4  ;# bugfix that <strong>-Tag appears before game list has started
set version 5.5  ;# change date-format of Spielbox to English
set version 5.6  ;# bugfix: reset all data before parsing BGG website
set version 5.7  ;# bugfix: do not overwrite with empty values
set version 5.8  ;# bugfix: parse booth number if several booths are given

# Initialaufruf:
# ./update_messe.tcl webseite.html > datei.csv
# Danach:
# ./update_messe.tcl webseite.html bgg.html datei.csv > neue_datei.csv

if { $argc < 1 } {
    puts "Error: Keine HTML-Datei angegeben!"
    exit 1
}

set webpage    [lindex $argv 0]
set webpageBGG ""
set csvfile    ""

# puts "Args: $argc $argv"

if { $argc > 1 } {
    if { [regexp {.html$} [lindex $argv 1]] } {
        set webpageBGG [lindex $argv 1]
        if { $argc > 2 } {
            if { [regexp {.csv$} [lindex $argv 2]] } {
                set csvfile [lindex $argv 2]
            } elseif { [string length [lindex $argv 2]] > 0 } {
                puts "Unknown file: [lindex $argv 2]"
                exit 1
            }
        }
    } elseif { [regexp {.csv$} [lindex $argv 1]] } {
        set csvfile [lindex $argv 1]
    } else {
        puts "Unknown file: [lindex $argv 1]"
        exit 1
    }
}

# -----------------------------------------------

proc readCsvEntry { line } {
    global g_csvVersion
    global g_newList
    global g_changedList
    global g_boothList
    global g_publisherList
    global g_gameList
    global g_playersList
    global g_ageList
    global g_durationList
    global g_languageList
    global g_priceList
    global g_prioList
    global g_remarkList
    global g_timestampList
    global g_bggLinkList
    global g_spielenLinkList
    global g_everythingChangedList
    global g_inCsvOnly

    if { $g_csvVersion >= 1.0 && $g_csvVersion < 2.0 } {
        # v1.0 - v1.1
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Prio,Remarks,LC,BGG"
        if { [regexp {^([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*)$} $line all new changed booth publisher game players age duration prio remarks timestamp bgglink ] } {
            set publisher [convertToSameFormat $publisher]
            set game [convertToSameFormat $game]
            set entry [createEntry $publisher $game]
            set g_newList($entry)       $new
            set g_changedList($entry)   $changed
            set g_boothList($entry)     $booth
            set g_publisherList($entry) $publisher
            set g_gameList($entry)      $game
            set g_playersList($entry)   $players
            set g_ageList($entry)       $age
            set g_durationList($entry)  $duration
            set g_languageList($entry)  {}
            set g_priceList($entry)     {}
            set g_prioList($entry)      $prio
            set g_remarkList($entry)    $remarks
            set g_timestampList($entry) $timestamp
            set g_bggLinkList($entry)   $bgglink
            set g_spielenLinkList($entry) {}
            set g_everythingChangedList($entry) {}
            set g_inCsvOnly($entry)     1
            if { [string compare $g_newList($entry) "X"] == 0 } {
                set g_newList($entry) "Y"
            }
        } else {
            error "Wrong CSV line: $line"
        }
    } elseif { $g_csvVersion >= 2.0 && $g_csvVersion < 3.0 } {
        # v2.0
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Prio,Remarks,Created,BGG"
        if { [regexp {^([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*)$} $line all new changed booth publisher game players age duration prio remarks timestamp ] } {
            set publisher [convertToSameFormat $publisher]
            set game [convertToSameFormat $game]
            set entry [createEntry $publisher $game]
            set g_newList($entry)       $new
            set g_changedList($entry)   $changed
            set g_boothList($entry)     $booth
            set g_publisherList($entry) $publisher
            set g_gameList($entry)      $game
            set g_playersList($entry)   $players
            set g_ageList($entry)       $age
            set g_durationList($entry)  $duration
            set g_languageList($entry)  {}
            set g_priceList($entry)     {}
            set g_prioList($entry)      $prio
            set g_remarkList($entry)    $remarks
            set g_timestampList($entry) $timestamp
            set g_bggLinkList($entry)   {}
            set g_spielenLinkList($entry) {}
            set g_everythingChangedList($entry) {}
            set g_inCsvOnly($entry)     1
            if { [string compare $g_newList($entry) "X"] == 0 } {
                set g_newList($entry) "Y"
            }
        } else {
            error "Wrong CSV line: $line"
        }
    } elseif { $g_csvVersion >= 3.0 && $g_csvVersion < 4.0 } {
        # v3.0
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Prio,Remarks,Created,BGG"
        if { [regexp {^([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*)$} $line all new changed booth publisher game players age duration language prio remarks timestamp bgglink ] } {
            set publisher [convertToSameFormat $publisher]
            set game [convertToSameFormat $game]
            set entry [createEntry $publisher $game]
            set g_newList($entry)       $new
            set g_changedList($entry)   $changed
            set g_boothList($entry)     $booth
            set g_publisherList($entry) $publisher
            set g_gameList($entry)      $game
            set g_playersList($entry)   $players
            set g_ageList($entry)       $age
            set g_durationList($entry)  $duration
            set g_languageList($entry)  $language
            set g_priceList($entry)     {}
            set g_prioList($entry)      $prio
            set g_remarkList($entry)    $remarks
            set g_timestampList($entry) $timestamp
            set g_bggLinkList($entry)   $bgglink
            set g_spielenLinkList($entry) {}
            set g_everythingChangedList($entry) {}
            set g_inCsvOnly($entry)     1
            if { [string compare $g_newList($entry) "X"] == 0 } {
                set g_newList($entry) "Y"
            }
        } else {
            error "Wrong CSV line: $line"
        }
    } elseif { $g_csvVersion >= 4.0 && $g_csvVersion < 5.0 } {
        # v4.0
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Prio,Remarks,Created,BGG,spielen.de"
        if { [regexp {^([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*)$} $line all new changed booth publisher game players age duration language prio remarks timestamp bgglink spielenlink ] } {
            set publisher [convertToSameFormat $publisher]
            set game [convertToSameFormat $game]
            set entry [createEntry $publisher $game]
            set g_newList($entry)       $new
            set g_changedList($entry)   $changed
            set g_boothList($entry)     $booth
            set g_publisherList($entry) $publisher
            set g_gameList($entry)      $game
            set g_playersList($entry)   $players
            set g_ageList($entry)       $age
            set g_durationList($entry)  $duration
            set g_languageList($entry)  $language
            set g_priceList($entry)     {}
            set g_prioList($entry)      $prio
            set g_remarkList($entry)    $remarks
            set g_timestampList($entry) $timestamp
            set g_bggLinkList($entry)   $bgglink
            set g_spielenLinkList($entry) $spielenlink
            set g_everythingChangedList($entry) {}
            set g_inCsvOnly($entry)     1
            if { [string compare $g_newList($entry) "X"] == 0 } {
                set g_newList($entry) "Y"
            }
        } else {
            error "Wrong CSV line: $line"
        }
    } elseif { $g_csvVersion >= 5.0 && $g_csvVersion < 6.0 } {
        # v5.0
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Price,Prio,Remarks,Created,BGG,spielen.de"
        if { [regexp {^([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*)$} $line all new changed booth publisher game players age duration language price prio remarks timestamp bgglink spielenlink ] } {
            set publisher [convertToSameFormat $publisher]
            set game [convertToSameFormat $game]
            set entry [createEntry $publisher $game]
            set g_newList($entry)       $new
            set g_changedList($entry)   $changed
            set g_boothList($entry)     $booth
            set g_publisherList($entry) $publisher
            set g_gameList($entry)      $game
            set g_playersList($entry)   $players
            set g_ageList($entry)       $age
            set g_durationList($entry)  $duration
            set g_languageList($entry)  $language
            set g_priceList($entry)     $price
            set g_prioList($entry)      $prio
            set g_remarkList($entry)    $remarks
            set g_timestampList($entry) $timestamp
            set g_bggLinkList($entry)   $bgglink
            set g_spielenLinkList($entry) $spielenlink
            set g_everythingChangedList($entry) {}
            set g_inCsvOnly($entry)     1
            if { [string compare $g_newList($entry) "X"] == 0 } {
                set g_newList($entry) "Y"
            }
        } else {
            error "Wrong CSV line: $line"
        }
    } else {
        error "readCsvEntry Unknown version: $g_csvVersion"
    }
}

proc checkCsvVersion { file } {
    global g_csvVersion
    gets $file line
    regexp {^v([^,]*),} $line all g_csvVersion
    #puts $g_csvVersion
    gets $file line
    if { $g_csvVersion >= 1.0 && $g_csvVersion < 2.0 } {
        # v1.x
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Prio,Remarks,LC,BGG"
        if { [string compare $line "N,C,Booth,Publisher,Game,Players,Age,Dur,Prio,Remarks,LC,BGG"] != 0 } {
            error "CSV-format does not match version $g_csvVersion: $line"
        }
    } elseif { $g_csvVersion >= 2.0 && $g_csvVersion < 3.0 } {
        # v2.x
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Prio,Remarks,Created"
        if { [string compare $line "N,C,Booth,Publisher,Game,Players,Age,Dur,Prio,Remarks,Created"] != 0 } {
            error "CSV-format does not match version $g_csvVersion: $line"
        }
    } elseif { $g_csvVersion >= 3.0 && $g_csvVersion < 4.0 } {
        # v3.x
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Prio,Remarks,Created,BGG"
        if { [string compare $line "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Prio,Remarks,Created,BGG"] != 0 } {
            error "CSV-format does not match version $g_csvVersion: $line"
        }
    } elseif { $g_csvVersion >= 4.0 && $g_csvVersion < 5.0 } {
        # v4.x
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Prio,Remarks,Created,BGG,spielen.de"
        if { [string compare $line "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Prio,Remarks,Created,BGG,spielen.de"] != 0 } {
            error "CSV-format does not match version $g_csvVersion: $line"
        }
    } elseif { $g_csvVersion >= 5.0 && $g_csvVersion < 6.0 } {
        # v5.x
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Price,Prio,Remarks,Created,BGG,spielen.de"
        if { [string compare $line "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Price,Prio,Remarks,Created,BGG,spielen.de"] != 0 } {
            error "CSV-format does not match version $g_csvVersion: $line"
        }
    } else {
        error "checkCsvVersion Unknown version: $g_csvVersion"
    }
}

proc readCsv { filename } {
    global g_csvVersion
    set file [open $filename r]
    checkCsvVersion $file
    while { [gets $file line] >= 0 } {
        readCsvEntry $line
    }
}

# -----------------------------------------------

proc convertUnicode { line } {
    regsub -all {,} $line {} line
    regsub -all {&amp;} $line {\&} line
    regsub -all { & } $line { and } line
    regsub -all {&#039;} $line {} line
    regsub -all {\?\?\?} $line {Fragezeichen} line
    regsub -all {\?} $line {} line
    regsub -all {–} $line {-} line
    regsub -all {´} $line {'} line
    regsub {^[ \t]*} $line {} line
    regsub {[ \t]*$} $line {} line
    regsub {               } $line {} line
    return $line
}

proc createEntry { publisher game } {
    set entry "$publisher,$game"
    # regsub -all {,} $entry {} entry
    regsub -all {\.} $entry {} entry
    regsub -all {!} $entry {} entry
    regsub -all {\?} $entry {} entry
    regsub -all {:} $entry {} entry
    regsub -all {'} $entry {} entry
    regsub -all {\-} $entry {} entry
    regsub -all { } $entry {} entry
    regsub -all {\(} $entry {} entry
    regsub -all {\)} $entry {} entry
    regsub -all {/} $entry {} entry
    regsub -all {­­­} $entry {} entry
    regsub -all {\+} $entry {} entry
    regsub -all {\+} $entry {} entry
    regsub {^[0-9]*} $entry {} entry
    return $entry
}

proc checkChanged { entry oldValue newValue hasChanged } {
    global g_everythingChangedList
    
    upvar $oldValue localOldValue
    upvar $newValue localNewValue
    upvar $hasChanged localHasChanged
    
    if { [info exists localNewValue] && [string length $localNewValue] > 0 } {
        if { [string compare $localOldValue $localNewValue] } {
            if { [string length $localOldValue] > 0 } {
                set localHasChanged 1
            }
            lappend g_everythingChangedList($entry) "<$localOldValue> -> <$localNewValue>"
            set localOldValue $localNewValue
        }
        unset localNewValue
    }
}

proc storeGame { } {

    global g_newList
    global g_changedList
    global g_boothList
    global g_publisherList
    global g_gameList
    global g_playersList
    global g_ageList
    global g_durationList
    global g_languageList
    global g_priceList
    global g_prioList
    global g_remarkList
    global g_timestampList
    global g_bggLinkList
    global g_spielenLinkList
    global g_everythingChangedList
    global g_inCsvOnly

    global g_publisher
    global g_booth
    global g_game
    global g_players
    global g_playtime
    global g_age
    global g_bgglink
    global g_spielenlink
    global g_timestamp

    global g_counter

    if { [info exists g_publisher] && [info exists g_game] } {
        set entry [createEntry $g_publisher $g_game]
        set isNew 0
        incr g_counter
        #puts "$g_counter $entry"
        if { ![info exists g_gameList($entry)] } {
            # Spiel existiert noch nicht in der Liste
            set isNew 1
            set g_newList($entry)        {}
            set g_changedList($entry)    {}
            set g_boothList($entry)      {}
            set g_publisherList($entry)  {}
            set g_gameList($entry)       {}
            set g_playersList($entry)    {}
            set g_durationList($entry)   {}
            set g_ageList($entry)        {}
            set g_languageList($entry)   {}
            set g_priceList($entry)      {}
            set g_prioList($entry)       {}
            set g_remarkList($entry)     {}
            set g_timestampList($entry)  {}
            set g_bggLinkList($entry)    {}
            set g_spielenLinkList($entry) {}
            set g_everythingChangedList($entry) {}
            set g_inCsvOnly($entry)      0
        } else {
            set g_inCsvOnly($entry)      0
        }

        set g_publisherList($entry) $g_publisher
        # Publisher nicht loeschen, da der nicht wiederholt wird!

        if { [info exists g_booth] && [string length $g_booth] > 0 } {
            if { [string compare $g_boothList($entry) $g_booth] } {
                if { ![string compare $g_boothList($entry) ""] } {
                    set hasChanged 1
                }
                set g_boothList($entry) $g_booth
            }
            # Booth nicht loeschen, da es nicht wiederholt wird
        }

        set g_gameList($entry) $g_game
        unset g_game

        set hasChanged 0
        checkChanged $entry g_playersList($entry) g_players hasChanged
        checkChanged $entry g_durationList($entry) g_playtime hasChanged
        checkChanged $entry g_ageList($entry) g_age hasChanged
        checkChanged $entry g_bggLinkList($entry) g_bgglink hasChanged
        checkChanged $entry g_spielenLinkList($entry) g_spielenlink hasChanged
        # checkChanged $entry g_timestampList($entry) g_timestamp hasChanged
        set g_timestampList($entry) [changeToEnglishTimestamp $g_timestamp]
        
        if { $isNew } {
            set g_newList($entry) "X"
        }
        # else lassen wir den vorherigen Eintrag stehen
        if { $hasChanged } {
            set g_changedList($entry) "X"
        }
    }
}

proc changeToEnglishTimestamp { timestamp } {
    # Wandle
    #     10.06.2015 09:00 Uhr
    # zu
    #     2015-06-10 09:00:00
    if { [regexp {([0-9][0-9])\.([0-9][0-9])\.([0-9][0-9][0-9][0-9]) ([0-9][0-9]:[0-9][0-9]) Uhr} $timestamp all day month year hour] } {
        set timestamp "$year-$month-$day $hour:00"
    } elseif { [regexp {[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9] [0-9][0-9]:[0-9][0-9]:[0-9][0-9]} $timestamp] } {
        # already in English format
    } else {
        error "Timestamp format incorrect: $timestamp"
    }
    
    return $timestamp
}

proc checkPublisher { line } {
    global g_publisher
    global g_booth
    global g_game
    global g_publisherStarted
    if { $g_publisherStarted } {
        #puts "$line"
        if { [regexp {<a.*class="publisherlink"[^>]*>([^<]*)</a>} $line all publisher] } {
            set publisher [convertUnicode $publisher] 
            set publisher [convertToSameFormat $publisher]
            #puts "> $publisher"
            set g_publisher $publisher
        } else {
            regsub -all {<[^>]*>} $line {} line
            set publisher [convertUnicode $line]
            set publisher [convertToSameFormat $publisher]
            if { [string length $publisher] > 0 } {
                #puts ">> $publisher"
                set g_publisher $publisher
            }
        }
        if { [info exists g_booth] } {
            unset g_booth
        }
    }
}

proc checkGameTitle { line } {
    global g_game
    global g_gameStarted
    global g_gameNameStarted
    if { $g_gameStarted && $g_gameNameStarted } {
        # puts "$line"
        regsub -all {<[^>]*>} $line {} line
        set game [convertUnicode $line]
        set game [convertToSameFormat $game]
        if { [string length $game] > 0 } {
            set g_game $game
        }
    }
}

proc checkPlayers { line } {
    global g_players
    global g_gameStarted
    if { $g_gameStarted } {
        regexp {, (.*) Spieler} $line all g_players
    }
}

proc checkPlaytime { line } {
    global g_playtime
    global g_gameStarted
    if { $g_gameStarted } {
        regexp {, Spieldauer ca. (.*) Minuten} $line all g_playtime
    }
}

proc checkAge { line } {
    global g_age
    global g_gameStarted
    if { $g_gameStarted } {
        regexp {ab (.*) Jahren} $line all g_age
    }
}

proc checkBooth { line } {
    global g_booth
    if { [regexp {\(Messestand: *([^\(\)]*) *\)} $line all listOfBooths] } {
        set g_booth {}
        set listOfBooths [convertUnicode $listOfBooths]
        regsub {und} $listOfBooths {/} listOfBooths
        # Mehrere Staende werden durch / getrennt.
        set listOfBooths [split $listOfBooths /]
        foreach booth $listOfBooths {
            # Angaben wie "01-F-100" in "1-F100" wandeln, weil kuerzer
            regsub {^0 *([0-9])- *([A-Z])-(.*)$} [string trim $booth] {\1-\2\3} booth
            if { [string length $booth] > 0 } {
                # puts "|$booth|"
                if { [string length $g_booth] > 0 } {
                    set g_booth "$g_booth;$booth"
                } else {
                    set g_booth "$booth"
                }
            }
        }
        #puts "|$g_booth|"
    }
}

proc checkBggLink { line } {
    global g_bgglink
    if { [regexp {<a href="([^"]*)"[^>]*>BGG</a>} $line all g_bgglink] } {  ;#"
        # puts $g_bgglink
    }
}

proc checkTimestamp { line } {
    global g_timestamp
    global g_gameStarted
    if { $g_gameStarted } {
        if { [regexp {<nobr>\[([^\]]*)\]</nobr>} $line all g_timestamp] } {
            # puts $g_timestamp
        }
    }
}

proc checkSpielenLink { line } {
    global g_spielenlink
    if { [regexp {<a href="([^"]*)"[^>]* class="info2">mehr Informationen[^<]*</a>} $line all g_spielenlink] } {  ;#"
        # puts $g_bgglink
    }
}

proc checkMode { line } {
    global g_publisherStarted
    global g_gameListStarted
    global g_gameStarted
    global g_gameNameStarted

    if { [regexp {<h2>} $line] } {
        if { $g_publisherStarted } {
            error "Zweiten Publisher über <h2> gefunden!"
        }
        set g_publisherStarted 1
    }
    if { [regexp {</h2>} $line] } {
        if { !$g_publisherStarted } {
            error "Publisher beendet über </h2> ohne Start!"
        }
        set g_publisherStarted 0
    }
    if { [regexp {<ul>} $line] } {
        if { $g_gameListStarted } {
            error "Zweite Spieleliste begonnen über <ul> gefunden!"
        }
        set g_gameListStarted 1
    }
    if { [regexp {</ul>} $line] } {
        if { !$g_gameListStarted } {
            error "Spieleliste beendet über </ul> ohne Start!"
        }
        set g_gameListStarted 0
    }
    if { [regexp {<li>} $line] } {
        if { !$g_gameListStarted } {
            error "Spieleliste hat noch nicht begonnen!"
        }
        if { $g_gameStarted } {
            error "Zweites Spiel begonnen über <li> gefunden!"
        }
        set g_gameStarted 1
    }
    if { [regexp {</li>} $line] } {
        if { !$g_gameListStarted } {
            error "Spieleliste hat noch nicht begonnen!"
        }
        if { !$g_gameStarted } {
            error "Spiel beendet über </li> ohne Start!"
        }
        set g_gameStarted 0
        storeGame
    }
    if { [regexp {<strong>} $line] } {
        if { !$g_gameListStarted } {
            # error "Spieleliste hat noch nicht begonnen!"
            # Auskommentiert, weil das inzwischen regulaer passieren kann!
        } else {
            if { !$g_gameStarted } {
                error "Spiel hat noch nicht begonnen!"
            }
            if { $g_gameNameStarted } {
                error "Zweiter Spielname begonnen über <strong> gefunden!"
            }
            set g_gameNameStarted 1
        }
    }
    if { [regexp {</strong>} $line] } {
        if { !$g_gameListStarted } {
            # error "Spieleliste hat noch nicht begonnen!"
            # Auskommentiert, weil das inzwischen regulaer passieren kann!
        } else {
            if { !$g_gameStarted } {
                error "Spiel hat noch nicht begonnen!"
            }
            if { !$g_gameNameStarted } {
                error "Spielname beendet über </strong> ohne Start!"
            }
            set g_gameNameStarted 0
        }
    }
}

proc createList { line } {
    # Wir erstellen aus der Zeile eine Liste mit den einzelnen Datensegmenten.
    set sep µ
    if { [regexp $sep $line] } {
        error "Zeile $line enthält Trennzeichen $sep."
    }
    regsub -all {<a>} $line "$sep<a>" line
    regsub -all {</a>} $line "</a>$sep" line
    regsub -all {<h2>} $line "$sep<h2>$sep" line
    regsub -all {</h2>} $line "$sep</h2>$sep" line
    regsub -all {<ul>} $line "$sep<ul>$sep" line
    regsub -all {</ul>} $line "$sep</ul>$sep" line
    regsub -all {<li>} $line "$sep<li>$sep" line
    regsub -all {</li>} $line "$sep</li>$sep" line
    regsub -all {<strong>} $line "$sep<strong>$sep" line
    regsub -all {</strong>} $line "$sep</strong>$sep" line
    regsub -all {<nobr>} $line "$sep<nobr>" line
    regsub -all {</nobr>} $line "</nobr>$sep" line
    regsub -all {&nbsp;} $line " " line
    regsub -all {^\s*$} $line "" line
    return [split $line $sep]
}

proc readHtmlSpielbox { filename } {
    global g_publisherStarted
    global g_gameListStarted
    global g_gameStarted
    global g_gameNameStarted
    global g_counter

    set g_publisherStarted 0
    set g_gameListStarted 0
    set g_gameStarted 0
    set g_gameNameStarted 0
    set g_counter 0
    
    set file [open $filename r]
    while { [gets $file line_from_file] >= 0 } {
        set list_of_lines [createList $line_from_file] 
        foreach line $list_of_lines {
            checkMode $line
            checkPublisher $line
            checkBooth $line
            checkGameTitle $line
            checkPlayers $line
            checkPlaytime $line
            checkAge $line
            checkBggLink $line
            checkTimestamp $line
            checkSpielenLink $line
        }
    }
}

proc convertToSameFormat { line } {
    
    # Publisher
    regsub {^かぼへる \(Kaboheru\)$} $line {Kaboheru} line
    regsub {^新ボードゲーム党 \(New Board Game Party\)$} $line {New Board Game Party} line
    regsub {^999 Games b\.v\.$} $line {999 Games} line
    regsub {^Abacusspiele$} $line {ABACUSSPIELE} line
    regsub {^Abbagames$} $line {ABBA Games} line
    regsub {^ADC Blackfire Entertainment GmbH$} $line {ADC Blackfire} line
    regsub {^ADC Blackfire Entertainment$} $line {ADC Blackfire} line
    regsub {^A-games \(Board Game Publisher\)$} $line {A-games} line
    regsub {^Amigo Spiel \+ Freizeit GmbH$} $line {AMIGO} line
    regsub {^AMIGO Spiel \+ Freizeit GmbH$} $line {AMIGO} line
    regsub {^Arclight Games$} $line {Arclight} line
    regsub {^Asmodee Deutschland$} $line {Asmodee} line
    regsub {^Backspindle Games Ltd.$} $line {Backspindle Games} line
    regsub {^Bezier Games Inc.$} $line {Bézier Games} line
    regsub {^Blackrock Editions$} $line {Blackrock Games} line
    regsub {^Blam!$} $line {BLAM!} line
    regsub {^BLAM !$} $line {BLAM!} line
    regsub {^Blue Cocker$} $line {Blue Cocker Games} line
    regsub {^Blue Orange \(EU\)$} $line {Blue Orange Games} line
    regsub {^Board&Dice$} $line {Board and Dice} line
    regsub {^Cool Mini Or Not$} $line {Cool Mini or Not} line
    regsub {^Competo \(Marektoy\)$} $line {Competo / Marektoy} line
    regsub {^Edigrafica Games$} $line {Edigráfica Games} line
    regsub {^Elzra$} $line {Elzra Corp.} line
    regsub {^Franjos Spieleverlag$} $line {franjos Spieleverlag} line
    regsub {^Gemblo$} $line {Gemblo Inc.} line
    regsub {^Gigamic International Games$} $line {Gigamic} line
    regsub {^GMT Games$} $line {GMT Games Inc.} line
    regsub {^HABA-Habermaaß GmbH$} $line {HABA} line
    regsub {^Haba-Habermaaß GmbH$} $line {HABA} line
    regsub {^HABA - Habermaaß GmbH$} $line {HABA} line
    regsub {^Hans im Glück Verlags-GmbH$} $line {Hans im Glück} line
    regsub {^Hans im Glück Verlag$} $line {Hans im Glück} line
    regsub {^Huch! and friends$} $line {HUCH! and friends} line
    regsub {^Iello$} $line {IELLO} line
    regsub {^JaponBrand$} $line {Japon Brand} line
    regsub {^Korea Boardgames co\. Ltd\.$} $line {Koreaboardgames} line
    regsub {^Korea Boardgames$} $line {Koreaboardgames} line
    regsub {^Kosmos$} $line {KOSMOS} line
    regsub {^Kyy Games$} $line {KYY Games} line
    regsub {^Lautapelit$} $line {Lautapelit.fi} line
    regsub {^Logis$} $line {LOGIS} line
    regsub {^Mage Company$} $line {MAGE Company} line
    regsub {^Mario Truant Verlag$} $line {Truant Spiele} line
    regsub {^Mayfair Games Inc.$} $line {Mayfair Games} line
    regsub {^Morning Players S.A.S.$} $line {Morning Players} line
    regsub {^Österreichisches Spiele Museum e\.V\.$} $line {Österreichisches Spiele Museum} line
    regsub {^Phalanx Games$} $line {PHALANX} line
    regsub {^Playmore Games Inc. \(Finnish Publisher\)$} $line {Playmore Games Inc.} line
    regsub {^Playthisone$} $line {PLAYthisONE} line
    regsub {^RandD Games$} $line {R\&D Games} line
    regsub {^RandR Games$} $line {R\&R Games} line
    regsub {^Ravensburger Spieleverlag GmbH$} $line {Ravensburger} line
    regsub {^Ravensburger Spieleverlag$} $line {Ravensburger} line
    regsub {^RedImp Games$} $line {REDIMP GAMES} line
    regsub {^Schmidt Spiele GmbH$} $line {Schmidt Spiele} line
    regsub {^Sit down$} $line {Sit Down!} line
    regsub {^Surprised Stare Games$} $line {Surprised Stare Games Ltd} line
    regsub {^Tasty Minstrel Games \(TMG\)$} $line {Tasty Minstrel Games} line
    regsub {^Utsuroi$} $line {UTSUROI} line
    regsub {^Voodoo Games \(II\)$} $line {Voodoo Games} line
    regsub {^Wattsalpoag$} $line {WizKids Games} line
    regsub {^WizKids$} $line {Wattsalpoag Games} line
    regsub {^We Build Smiles \(WBS\) Games$} $line {WBS Games} line
    regsub {^Yay! Games$} $line {YAY! Games} line

    # Spiele
    regsub {^動物夜怕怕 \(Animals Frightening Night!\)$} $line {Animals Frightening Night!} line
    regsub {^1844/54$} $line {1844/1854} line
    regsub {^7 Ronin: Deluxe$} $line {7 Ronin} line
    regsub {^8\*28$} $line {8-28} line
    regsub {^A Study in Emerald \(second edition\)$} $line {A Study in Emerald} line
    regsub {^Abenteuerland$} $line {Adventure Land} line
    regsub {^Agents of Smersh: Showdown!$} $line {Agents of SMERSH: Showdown!} line
    regsub {^Alte Dunkle Dinge$} $line {Ancient Terrible Things} line
    regsub {^Arcadia Quest: Jenseits der Gruft$} $line {Arcadia Quest: Beyond the Grave} line
    regsub {^Aya$} $line {AYA} line
    regsub {^Bang! Dodge City$} $line {BANG! Dodge City} line
    regsub {^Bang! Expansion Pack$} $line {BANG! Expansion Pack} line
    regsub {^Bang! Gold Rush$} $line {BANG! Gold Rush} line
    regsub {^Bang! The Duel$} $line {BANG! The Duel} line
    regsub {^Barking up the wrong tree$} $line {Barking Up The Wrong Tree} line
    regsub {^Boost$} $line {Rally Up!} line
    regsub {^Brettspiel-Adventskalender$} $line {Brettspiel Adventskalender 2015} line
    regsub {^Burg Schlummerschatz$} $line {Sleepy Castle} line
    regsub {^Cacao: Vulkane$} $line {Cacao: Volcanoes} line
    regsub {^Carcassonne: Über Stock und Stein$} $line {Carcassonne: Over Hill and Dale} line
    regsub {^Cash and Guns: More Cash n More Guns$} $line {Ca$h 'n Guns: More Cash 'n More Guns} line
    regsub {^Caverna: Wasser Mini-Erweiterung$} $line {Caverna: Mini Expansion} line
    regsub {^Citrus: Neue Landschaftsplättchen - Promoplättchen$} $line {Citrus: Neue Landschaftsplättchen} line
    regsub {^Clacks: A Discworld Game$} $line {Clacks: A Discworld Board Game} line
    regsub {^Club der Verschwender$} $line {The Prodigals Club} line
    regsub {^Colt Express: Postkutsche and Pferde$} $line {Colt Express: Horses and Stagecoach} line
    regsub {^Consiglio dei Quattro$} $line {Council of Four} line
    regsub {^Crabz$} $line {Crab Stack} line
    regsub {^Das allerbeste Baumhaus$} $line {Best Treehouse Ever} line
    regsub {^Das Konzil der Vier$} $line {Council of Four} line
    regsub {^Der Herr des Eisgartens$} $line {The Lord of the Ice Garden} line
    regsub {^Die Baumeister Antike$} $line {The Builders: Antiquity} line
    regsub {^Die blutige Herberge$} $line {The Bloody Inn} line
    regsub {^Die Holde Isolde$} $line {Die holde Isolde} line
    regsub {^Die Piraten der 7 Weltmeere$} $line {Pirates of the 7 Seas} line
    regsub {^Die Schlösser des König Ludwig$} $line {Castles of Mad King Ludwig} line
    regsub {^Die unüblichen Verdächtigen$} $line {Unusual Suspects} line
    regsub {^Die Zähne des Drachen$} $line {Dragon Teeth Washer} line
    regsub {^Dixit 6: Memories$} $line {Dixit: Memories} line
    regsub {^Dominion: Abenteuer$} $line {Dominion: Adventures} line
    regsub {^Dreadball Xtreme$} $line {DreadBall Xtreme} line
    regsub {^Duell: Spiel mit mir das Spiel vom Tod$} $line {Duel: Once Upon a Game in the West} line
    regsub {^Dungeon Saga: Im Auftrag des Zwergenkönigs$} $line {Dungeon Saga: Dwarf King's Quest} line
    regsub {^Eine Studie in Smaragdgrün$} $line {A Study in Emerald} line
    regsub {^Elfenland de luxe$} $line {Elfenroads} line
    regsub {^Eclipse:Schatten aus dem Riss$} $line {Eclipse: Shadow of the Rift} line
    regsub {^Escape: Zombie City - Expansion 1 - The Survivor Chronicles$} $line {Escape: Zombie City - The Survivor Chronicles} line
    regsub {^Escape: Erweiterung 3 - Traps$} $line {Escape: Traps} line
    regsub {^Eclipse: Black Hole - Mini-Erweiterung$} $line {Eclipse: Black Hole} line
    regsub {^FishnStones: Heron - Promokarte$} $line {Fish'n'Stones: Heron} line
    regsub {^Fitz Stibitz$} $line {The Sneaky Snacky Squirrel Game} line
    regsub {^Fritz Stibitz$} $line {The Sneaky Snacky Squirrel Game} line
    regsub {^Flash Point - Ehre und Pflicht$} $line {Flash Point: Fire Rescue - Honor and Duty} line
    regsub {^Flash Point - Extreme Gefahr$} $line {Flash Point: Fire Rescue - Extreme Danger} line
    regsub {^Fleet Commander: Beyond the Gate$} $line {Fleet Commander: 2 - Beyond the Gate} line
    regsub {^Funkenschlag: Die Aktiengesellschaften$} $line {Power Grid: The Stock Companies} line
    regsub {^Glenn Drover's Empires: Age of Discovery - Deluxe Edition$} $line {Empires: Age of Discovery} line
    regsub {^Goldene Zeitalter$} $line {The Golden Ages} line
    regsub {^GSBM$} $line {GSBM - Great Strategy of Battle Monster} line
    regsub {^Hope$} $line {HOPE} line
    regsub {^Hoyuk: Anatolia$} $line {Höyük: Anatolia} line
    regsub {^Idol Conclave \+ Million hit Chart$} $line {Idol Conclave} line
    regsub {^Iki$} $line {IKI} line
    regsub {^Insekten Hotel$} $line {Insect Hotel} line
    regsub {^Insekten-Hotel$} $line {Insect Hotel} line
    regsub {^Isle of Skye: From Chieftain to King$} $line {Isle of Skye} line
    regsub {^Istanbul: Mokka und Bakschisch$} $line {Istanbul: Mocha and Baksheesh} line
    regsub {^Katakomben$} $line {Catacombs} line
    regsub {^Katakomben: Die Höhle von Soloth$} $line {Catacombs: Cavern of Soloth} line
    regsub {^Kerflip!$} $line {KerFlip!} line
    regsub {^Keyflower: Schweinebuchte - Promoplättchen$} $line {Keyflower: Pig Shelter} line
    regsub {^Klask$} $line {KLASK} line
    regsub {^Klamotten Klamauk$} $line {Raccoon Rumpus} line
    regsub {^Kumo Hogosha$} $line {KUMO Hogosha} line
    regsub {^Kune v Lakia: A chronicle of a royal lapine divorce foretold$} $line {KUNE v LAKIA: A Chronicle Of A Royal Lapine Divorce Foretold} line
    regsub {^La Torre dei Monstri$} $line {The Tower} line
    regsub {^Ligretto$} $line {Ligretto: Star Wars Rebels} line
    regsub {^Little Red Riding Hood$} $line {Tales and Games: Little Red Riding Hood} line
    regsub {^London after Midnight$} $line {London After Midnight} line
    regsub {^Lost Legacy: Todesschwert and Weißgoldturm} $line {Lost Legacy: Second Chronicle - Vorpal Sword and Whitegold Spire} line
    regsub {^Mage Knight - Das Brettspiel$} $line {Mage Knight Board Game} line 
    regsub {^Mage Wars: Forged in Fire - Spell Tome Expansion$} $line {Mage Wars Arena: Forged in Fire} line 
    regsub {^Mage Wars Arena: Flammenschmiede$} $line {Mage Wars Arena: Forged in Fire} line 
    regsub {^Merkurya$} $line {Safe Breaker} line 
    regsub {^Monster Menü$} $line {Monster Menu} line 
    regsub {^Monsterturm$} $line {The Tower} line 
    regsub {^Munchkin: Jäger des Schatzes$} $line {Munchkin Treasure Hunt} line
    regsub {^Nebel über Valskyrr$} $line {Mistfall} line 
    regsub {^Njet!$} $line {Nyet!} line
    regsub {^Peloponnes Kartenspiel$} $line {Peloponnes Card Game} line
    regsub {^Project: Elite$} $line {Project: ELITE} line
    regsub {^Qwirkle$} $line {Qwirkle: Star Wars} line
    regsub {^RevoltAAA$} $line {RevoltaaA} line
    regsub {^Rumms$} $line {Cube Quest} line
    regsub {^Ryu$} $line {RYŪ} line
    regsub {^Savanne$} $line {Sawanna} line
    regsub {^Scape$} $line {SCAPE} line
    regsub {^Schatzjäger$} $line {Treasure Hunter} line
    regsub {^Schinderhannes: Schwarzer Peter - Promokarte$} $line {Schinderhannes: Schwarzer Peter} line
    regsub {^Schubidubi Schabernack$} $line {Silly Shenanigans} line
    regsub {^Shakespeare: Prospero - Promokarte$} $line {Shakespeare: Prospero} line
    regsub {^Snowdonia: Card 31 of 30 - Promokarten$} $line {Snowdonia: Card 31 of 30} line
    regsub {^Spellenkart: Kesse Rüben$} $line {Kesse Rüben} line
    regsub {^Star Munchkin 3: Diplomatische Immunität$} $line {Star Munchkin 3: Diplomatic Impunity} line
    regsub {^Star Wars Rebels: Angriff der Rebellen$} $line {Star Wars: Angriff der Rebellen} line
    regsub {^Steam Ship Company \(Prototyp\)$} $line {Steam Ship Company} line
    regsub {^Stronghold: Zweite Edition$} $line {Stronghold} line
    regsub {^Stronghold \(second edition\)$} $line {Stronghold} line
    regsub {^Suburbia 5★$} $line {Suburbia: 5 Stars} line
    regsub {^Suburbia: 5 Sterne$} $line {Suburbia: 5 Stars} line
    regsub {^Tash-Kalar: Arena of Legends - Nethervoid$} $line {Tash-Kalar: Nethervoid} line
    regsub {^Teamplay$} $line {Team Play} line
    regsub {^The Game On Fire$} $line {The Game on Fire} line
    regsub {^The Dome$} $line {the DOME} line
    regsub {^The Gate$} $line {the GATE} line
    regsub {^Through the Ages - Zweite Auflage$} $line {Through the Ages: A New Story of Civilization} line
    regsub {^Tichu Sonderausgabe$} $line {Tichu} line
    regsub {^Tides of Time: Im Strom der Zeit$} $line {Tides of Time} line
    regsub {^Time Stories$} $line {T.I.M.E. Stories} line
    regsub {^Time Stories: Der Marcy-Fall$} $line {T.I.M.E. Stories: The Marcy Case} line
    regsub {^T.I.M.E. Stories: Der Marcy-Fall$} $line {T.I.M.E. Stories: The Marcy Case}
    regsub {^Top and Flop$} $line {Wits and Wagers} line
    regsub {^Tumult Royal$} $line {Tumult Royale} line
    regsub {^Wharfside$} $line {Fleet: Wharfside} line
    regsub {^Winziges Weltall$} $line {Tiny Epic Galaxies} line
    regsub {^Wo ist das Gehirn$} $line {Give Me the Brain!} line
    regsub {^Wunsch Maschine$} $line {Wunschmaschine} line
    regsub {^Zombie Würfel 3: Schulbus$} $line {Zombie Dice 3: School Bus} line
    regsub {^Zombie Würfel Deluxe$} $line {Zombie Dice} line
    regsub {^Zug um Zug: Deutschland - Deutschland 1902$} $line {Zug um Zug: Deutschland 1902} line
    regsub {^Ticket to Ride: Orient Express$} $line {Zug um Zug: Orient Express} line
    regsub {^Ticket to Ride Map Collection: Volume 5 - United Kingdom and Pennsylvania$} $line {Zug um Zug: Vereinigtes Königreich} line

    return $line
}

proc checkPublisherBGG { line } {
   global g_publisher
   if { [regexp {Board Game Publisher: <a.[^>]*>([^<]*)</a>} $line all publisher] } {
        set publisher [convertUnicode $publisher]
        set publisher [convertToSameFormat $publisher]
        # puts "> $publisher $line"
        set g_publisher $publisher
    }
}

proc checkGameTitleBGG { line } {
    global g_game
    if { [regexp {Board Game: <a [^>]*>([^<]*)</a>} $line all game] ||
         [regexp {Board Game Accessory: <a [^>]*>([^<]*)</a>} $line all game] } {
        set game [convertUnicode $game]
        set game [convertToSameFormat $game]
        # puts "+ $game $all"
        if { [string length $game] > 0 } {
            set g_game $game
        }
    }
}

proc checkBggLinkBGG { line } {
    global g_bgglink
    if { [regexp {Board Game[^:]*: <a [^h]*href="([^"]*)".*</a>} $line all link] } {
        set g_bgglink "https://boardgamegeek.com$link"
        # puts "$g_bgglink"
    }
}
#" ; Kommentar wegen Syntax-Highlighting-Fehler in Geany

proc checkTimestampBGG { line } {
    global g_timestamp
    if { [regexp {added by.*on ([^<]*)</span>} $line all g_timestamp] } {
        # TODO: Konvertiere "2015-08-10 11:56:07" in "10.08.2015 11:56"
        # puts "$g_timestamp"
        return 1
    }
    return 0
}

proc readHtmlBGG { filename } {
    global g_publisher
    global g_booth
    global g_game
    global g_players
    global g_playtime
    global g_age
    global g_bgglink
    global g_spielenlink
    global g_timestamp
    set g_publisher {}
    set g_booth {}
    set g_game {}
    set g_players {}
    set g_playtime {}
    set g_age {}
    set g_bgglink {}
    set g_spielenlink {}
    set g_timestamp {}
    
    global g_counter
    set g_counter 0
    set file [open $filename r]
    while { [gets $file line_from_file] >= 0 } {
        checkPublisherBGG $line_from_file
        checkGameTitleBGG $line_from_file
        checkBggLinkBGG   $line_from_file
        if { [checkTimestampBGG $line_from_file] } {
            storeGame
        }
    }
}

# -----------------------------------------------

proc printEntries { } {

    global g_newList
    global g_changedList
    global g_boothList
    global g_publisherList
    global g_gameList
    global g_playersList
    global g_ageList
    global g_durationList
    global g_languageList
    global g_priceList
    global g_prioList
    global g_remarkList
    global g_timestampList
    global g_bggLinkList
    global g_spielenLinkList
    global g_everythingChangedList
    global g_inCsvOnly
    global version

    puts "v$version,,,,,,,,," ;#,,,Extracted from:,http://partner.spielen.de/spielbox/messeneuheiten/verlag/2015/essen,,,,,,,,,
    puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Price,Prio,Remarks,Created,BGG,spielen.de"

    foreach entry [lsort -nocase -ascii [array names g_newList]] {
        puts -nonewline $g_newList($entry)
        puts -nonewline ","
        if { !$g_inCsvOnly($entry) } {
            puts -nonewline $g_changedList($entry)
        } else {
            if { [string first "BGG" $g_remarkList($entry)] == -1 } { 
                puts -nonewline "D"
            }
        }
        puts -nonewline ","
        puts -nonewline $g_boothList($entry)
        puts -nonewline ","
        puts -nonewline $g_publisherList($entry)
        puts -nonewline ","
        puts -nonewline $g_gameList($entry)
        puts -nonewline ","
        puts -nonewline $g_playersList($entry)
        puts -nonewline ","
        puts -nonewline $g_ageList($entry)
        puts -nonewline ","
        puts -nonewline $g_durationList($entry)
        puts -nonewline ","
        puts -nonewline $g_languageList($entry)
        puts -nonewline ","
        puts -nonewline $g_priceList($entry)
        puts -nonewline ","
        puts -nonewline $g_prioList($entry)
        puts -nonewline ","
        puts -nonewline $g_remarkList($entry)
        puts -nonewline ","
        puts -nonewline $g_timestampList($entry)
        puts -nonewline ","
        puts -nonewline $g_bggLinkList($entry)
        puts -nonewline ","
        puts -nonewline $g_spielenLinkList($entry)
        #puts -nonewline " "
        #puts -nonewline $g_everythingChangedList($entry)
        puts ""
    }
}

# -----------------------------------------------

if { [string length $csvfile] > 0 } {
    readCsv $csvfile
}

readHtmlSpielbox $webpage
readHtmlBGG $webpageBGG

printEntries





